package org.ioc.commons.impl.gwt.client.flowcontrol.eventbus;

import org.ioc.commons.flowcontrol.eventbus.IsEvent;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.web.bindery.event.shared.EventBus;
import com.google.web.bindery.event.shared.HandlerRegistration;

class GwtEventFiredEvent<E extends Enum<? extends IsEvent>> extends GwtEvent<GwtEventFiredEvent.Handler> {

	public static HandlerRegistration register(EventBus eventBus, GwtEventFiredEvent.Handler handler) {
		return eventBus.addHandler(TYPE, handler);
	}

	public static <E extends Enum<? extends IsEvent>> void fire(EventBus eventBus, E event, Object source,
			Object... params) {
		if (TYPE != null) {
			GwtEventFiredEvent<E> gwtEvent = new GwtEventFiredEvent<E>(event, params);
			eventBus.fireEventFromSource(gwtEvent, source);
		}
	}

	private E event;
	private Object[] params;

	GwtEventFiredEvent(E event, Object... params) {
		this.event = event;
		this.params = params;
	}

	/**
	 * Implemented by objects that handle {@link GwtEventFiredEvent}.
	 */
	public interface Handler extends EventHandler {
		void onEventFired(GwtEventFiredEvent<? extends Enum<? extends IsEvent>> event);
	}

	/**
	 * The event type.
	 */
	public static final Type<GwtEventFiredEvent.Handler> TYPE = new Type<GwtEventFiredEvent.Handler>();

	@Override
	public final Type<GwtEventFiredEvent.Handler> getAssociatedType() {
		return TYPE;
	}

	@Override
	protected void dispatch(GwtEventFiredEvent.Handler handler) {
		handler.onEventFired(this);
	}

	public E getEvent() {
		return event;
	}
	
	public Object[] getParams() {
		return params;
	}
}
