package org.ioc.commons.impl.gwt.client.flowcontrol.operationmanager;

import org.ioc.commons.flowcontrol.operationmanager.IsOperation;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.web.bindery.event.shared.EventBus;
import com.google.web.bindery.event.shared.HandlerRegistration;

class OperationBeginningEvent<O extends Enum<? extends IsOperation>> extends GwtEvent<OperationBeginningEvent.Handler> {

	public static HandlerRegistration register(EventBus eventBus, OperationBeginningEvent.Handler handler) {
		return eventBus.addHandler(TYPE, handler);
	}

	public static <O extends Enum<? extends IsOperation>> void fire(EventBus eventBus, O operation, Object source) {
		if (TYPE != null) {
			OperationBeginningEvent<O> event = new OperationBeginningEvent<O>(operation);
			eventBus.fireEventFromSource(event, source);
		}
	}

	private O operation;

	OperationBeginningEvent(O operation) {
		this.operation = operation;
	}

	/**
	 * Implemented by objects that handle {@link OperationBeginningEvent}.
	 */
	public interface Handler extends EventHandler {
		void onOperationBeginning(OperationBeginningEvent<? extends Enum<? extends IsOperation>> event);
	}

	/**
	 * The event type.
	 */
	public static final Type<OperationBeginningEvent.Handler> TYPE = new Type<OperationBeginningEvent.Handler>();

	@Override
	public final Type<OperationBeginningEvent.Handler> getAssociatedType() {
		return TYPE;
	}

	@Override
	protected void dispatch(OperationBeginningEvent.Handler handler) {
		handler.onOperationBeginning(this);
	}

	public O getOperation() {
		return operation;
	}
}
