package org.ioc.commons.impl.gwt.client.flowcontrol.operationmanager;

import org.ioc.commons.flowcontrol.operationmanager.IsOperation;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.web.bindery.event.shared.EventBus;
import com.google.web.bindery.event.shared.HandlerRegistration;

class OperationFinishedEvent<O extends Enum<? extends IsOperation>> extends GwtEvent<OperationFinishedEvent.Handler> {

	public static HandlerRegistration register(EventBus eventBus, OperationFinishedEvent.Handler handler) {
		return eventBus.addHandler(TYPE, handler);
	}

	public static <O extends Enum<? extends IsOperation>> void fire(EventBus eventBus, O operation, boolean success, Object source) {
		if (TYPE != null) {
			OperationFinishedEvent<O> event = new OperationFinishedEvent<O>(operation, success);
			eventBus.fireEventFromSource(event, source);
		}
	}

	private O operation;
	private boolean success;

	OperationFinishedEvent(O operation, boolean success) {
		this.operation = operation;
		this.success = success;
	}

	/**
	 * Implemented by objects that handle {@link OperationFinishedEvent}.
	 */
	public interface Handler extends EventHandler {
		void onOperationFinished(OperationFinishedEvent<? extends Enum<? extends IsOperation>> event);
	}

	/**
	 * The event type.
	 */
	public static final Type<OperationFinishedEvent.Handler> TYPE = new Type<OperationFinishedEvent.Handler>();

	@Override
	public final Type<OperationFinishedEvent.Handler> getAssociatedType() {
		return TYPE;
	}

	@Override
	protected void dispatch(OperationFinishedEvent.Handler handler) {
		handler.onOperationFinished(this);
	}

	public O getOperation() {
		return operation;
	}
	
	public boolean isSuccess() {
		return success;
	}
}
