package org.ioc.commons.impl.gwt.client.ui;

import org.ioc.commons.flowcontrol.actioncontroller.ActionController;
import org.ioc.commons.flowcontrol.actioncontroller.ActionControllerBinder;
import org.ioc.commons.flowcontrol.actioncontroller.IsExternalAction;
import org.ioc.commons.flowcontrol.eventbus.EventBus;
import org.ioc.commons.flowcontrol.eventbus.EventBusBinder;
import org.ioc.commons.flowcontrol.eventbus.IsEvent;
import org.ioc.commons.flowcontrol.operationmanager.IsOperation;
import org.ioc.commons.flowcontrol.operationmanager.OperationManager;
import org.ioc.commons.flowcontrol.operationmanager.OperationManagerBinder;
import org.ioc.commons.impl.gwt.client.flowcontrol.GwtFlowController;
import org.ioc.commons.ui.HasPosition;
import org.ioc.commons.ui.IsDisplay;
import org.ioc.commons.ui.IsPresenter;
import org.ioc.commons.ui.IsResizable;

import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.user.client.ui.Composite;
import com.google.web.bindery.event.shared.SimpleEventBus;

public abstract class DisplayComposite<E extends Enum<? extends IsEvent>, O extends Enum<? extends IsOperation>, A extends Enum<? extends IsExternalAction>, P extends IsPresenter>
		extends Composite implements IsDisplay<E, O, A>, IsResizable, HasPosition {
	private com.google.web.bindery.event.shared.EventBus gwtEventBus = new SimpleEventBus();

	protected final GwtFlowController<E, O, A> flowController = new GwtFlowController<E, O, A>(this.gwtEventBus);
	protected final EventBus<E> eventbus = flowController.eventBus();
	protected final OperationManager<O> operationManager = flowController.operationManager();
	protected final ActionController<A> actionController = flowController.actionController();

	protected P presenter;

	@Override
	protected void onLoad() {
		super.onLoad();
		if (this.presenter != null) {
			this.presenter.onInitDisplay();
		}
	}

	final public EventBusBinder<E> eventBus() {
		return this.eventbus;
	}

	final public OperationManagerBinder<O> operationManager() {
		return this.operationManager;
	}

	final public ActionControllerBinder<A> actionController() {
		return this.actionController;
	}

	@Override
	public int getWidth() {
		return super.getOffsetWidth();
	}

	@Override
	public int getHeight() {
		return super.getOffsetHeight();
	}

	@Override
	public int getX() {
		return super.getAbsoluteTop();
	}

	@Override
	public int getY() {
		return super.getAbsoluteLeft();
	}

	@Override
	public void setWidth(int width) {
		super.setWidth(width + "px");
	}

	@Override
	public void setHeight(int height) {
		super.setHeight(height + "px");
	}

	/**
	 * Utility method for checking if the user pressed ENTER key
	 * 
	 * @param evt
	 *            Key press event
	 * @return true if ENTER was pressed; false otherwise
	 */
	protected boolean enterPressed(KeyPressEvent evt) {
		char charCode = evt.getCharCode();
		return (charCode == '\r');
	}
}
