package org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory;

import java.util.HashMap;
import java.util.Map;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFDaoTools;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RequestQueue;
import org.ioc.commons.integration.dataaccess.dao.DaoTools;
import org.ioc.commons.integration.dataaccess.dao.DataManager;
import org.ioc.commons.integration.dataaccess.dao.IsDaoFactory;
import org.ioc.commons.integration.dataaccess.dao.Tx;

import com.google.web.bindery.requestfactory.shared.BaseProxy;
import com.google.web.bindery.requestfactory.shared.RequestFactory;

public abstract class RFDataManager<D extends IsDaoFactory, F extends RequestFactory> implements DataManager<D> {

	protected final Map<String, RequestQueue> queueRequests = new HashMap<String, RequestQueue>();
	protected final Map<BaseProxy, Tx<D>> bindedProxiesMap = new HashMap<BaseProxy, Tx<D>>();

	protected F dataFactory;

	private D noTxDaoFactory;

	public RFDataManager() {
	}

	public final void initialize(F dataFactory) {
		this.dataFactory = dataFactory;
	}

	public final boolean isInitialized() {
		return (this.dataFactory != null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.grupit.module.user.web.gwt.client.integration.dao.DaoManager#
	 * openTransaction(java.lang.String)
	 */
	@Override
	public abstract Tx<D> openTransaction(String transactionName);

	@Override
	public final D daoFactory() {
		if (this.noTxDaoFactory == null) {
			this.noTxDaoFactory = newNonTransactionalDaoFactory();
		}
		return noTxDaoFactory;
	}

	protected abstract D newNonTransactionalDaoFactory();

	public final F getDataFactory() {
		return dataFactory;
	}

	@Override
	public final boolean hasQueuePendingRequests(String queueName) {
		RequestQueue rq = queueRequests.get(queueName);

		if (rq != null) {
			return rq.hasPendingRequests();
		}

		return false;
	}

	@Override
	public final <P> Tx<D> getTransactionOwner(P dataObject) {
		return bindedProxiesMap.get(dataObject);
	}
	
	@Override
	public DaoTools daoTools() {
		return RFDaoTools.get();
	}
}
