package org.ioc.commons.impl.gwt.integration.client.service.rpc.rebind.internal;

import org.ioc.commons.flowcontrol.operationmanager.IsOperation;
import org.ioc.commons.flowcontrol.operationmanager.OperationManagerTrigger;
import org.ioc.commons.integration.service.ServiceCallback;
import org.ioc.commons.integration.service.ServiceOperationManager;

public class RpcServiceOperationManagerWrapper<T> implements ServiceOperationManager<T> {

	public static class OperationTrigger<O extends Enum<? extends IsOperation>> {
		private O operation;
		private OperationManagerTrigger<O> trigger;

		public OperationTrigger(O operation, OperationManagerTrigger<O> trigger) {
			this.operation = operation;
			this.trigger = trigger;
		}

		public void operationBeginning() {
			this.trigger.operationBeginning(operation);
		}

		public void operationFinished(boolean success) {
			this.trigger.operationFinished(operation, success);
		}
	}

	private OperationTrigger<? extends Enum<? extends IsOperation>> operaTrigger;

	private ServiceCallback<T> callback;

	public RpcServiceOperationManagerWrapper() {
		// Nothing
	}

	@Override
	public ServiceOperationManager<T> to(ServiceCallback<T> callback) {
		this.callback = callback;
		return this;
	}

	@Override
	public <O extends Enum<? extends IsOperation>> ServiceOperationManager<T> performing(O operation,
			OperationManagerTrigger<O> trigger) {
		return performing(new OperationTrigger<O>(operation, trigger));
	}

	private <O extends Enum<? extends IsOperation>> ServiceOperationManager<T> performing(
			OperationTrigger<O> operationTrigger) {
		this.operaTrigger = operationTrigger;
		return this;

	}

	public OperationTrigger<? extends Enum<? extends IsOperation>> getOperaTrigger() {
		return operaTrigger;
	}

	public ServiceCallback<T> getCallback() {
		return callback;
	}

}
