package org.ioc.commons.impl.gwt.rebind.dataaccess.requestfactory.generator;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFBaseDao;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFBaseEntityDao;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFBaseIdImpl;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFDaoFactory;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFDaoOperationManagerWrapper;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFDaoReceiver;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFSubscriptor;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RequestProvider;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RequestQueue;
import org.ioc.commons.integration.dataaccess.dao.DaoOperationManager;
import org.ioc.commons.integration.dataaccess.dao.IsEntityId;
import org.ioc.commons.integration.dataaccess.dao.Subscriptor;
import org.ioc.commons.utils.Format;

import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.TreeLogger.Type;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.JMethod;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.core.ext.typeinfo.JParameter;
import com.google.gwt.core.ext.typeinfo.JParameterizedType;
import com.google.gwt.core.ext.typeinfo.JType;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;
import com.google.web.bindery.requestfactory.shared.EntityProxyId;

public class RFDaoGenerator extends RFGeneratorBase {

	private JClassType dao;
	private boolean bugWarned;

	public RFDaoGenerator(JClassType dao) {
		this.dao = dao;
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);

		String rfDaoGeneratedSimpleClassname = typeName + RFDataManagerGenerator.GENERATED_CLASS_SUFIX;
		String rfDaoGeneratedFullClassname = rfDataManagerType.getPackage().getName() + "."
				+ rfDaoGeneratedSimpleClassname;

		if (entityDaos.contains(dao)) {
			logger.log(Type.INFO, Format.substitute("Generating entity dao {0}...", rfDaoGeneratedSimpleClassname));

			writeEntityDao(logger, context, rfDaoGeneratedSimpleClassname);
		} else {
			logger.log(Type.INFO, Format.substitute("Generating plain dao {0}...", rfDaoGeneratedSimpleClassname));

			writePlainDao(logger, context, rfDaoGeneratedSimpleClassname);
		}

		return rfDaoGeneratedFullClassname;
	}

	protected void writeEntityDao(TreeLogger logger, GeneratorContext context, String rfDaoGeneratedSimpleClassname)
			throws UnableToCompleteException {
		SourceWriter sw = getRFEntityDaoSourceWriter(logger, context, rfDaoGeneratedSimpleClassname);

		if (sw != null) {

			DaoMapping daoMapping = this.getDaoMapping(logger);

			sw.println("public %s(RequestProvider<%s> useProvider, Map<String, RequestQueue> queueRequests) {",
					rfDaoGeneratedSimpleClassname, requestFactoryClass.getName());
			sw.println("	super(useProvider, queueRequests);");
			sw.println("}");
			sw.println();
			sw.println("@Override");
			sw.println("public %s getRequest(RequestProvider<%s> p) {", daoMapping.request.getName(),
					requestFactoryClass.getName());
			sw.println("	return p.getRequest(%s.class);", daoMapping.request.getName());
			sw.println("}");
			sw.println();
			sw.println("@Override");
			sw.println("public %s create() {", daoMapping.proxy.getName());
			sw.println("	return createInternal(%s.class);", daoMapping.proxy.getName());
			sw.println("}");
			sw.println();

			RFIdGenerator idGenerator = new RFIdGenerator(daoMapping);
			String idImplClassname = idGenerator.generate(logger, context,
					Format.substitute("RF{0}", daoMapping.id.getSimpleSourceName()));

			sw.println("@SuppressWarnings(\"unchecked\")");
			sw.println("@Override");
			sw.println("public %s getId(Object obj) {", daoMapping.id.getName());
			sw.println("	if (obj == null) {");
			sw.println("		return null;");
			sw.println("	} else if (obj instanceof %s) {", daoMapping.proxy.getName());
			sw.println("		return new %s(((%s) obj).stableId());", idImplClassname, daoMapping.proxy.getName());
			sw.println("	} else if (obj instanceof EntityProxyId<?>) {");
			sw.println("		return new %s(((EntityProxyId<%s>) obj));", idImplClassname, daoMapping.proxy.getName());
			sw.println("	}");
			sw.println();
			sw.println("	throw new IllegalArgumentException(Format.substitute(");
			sw.println("			\"The object '{0}' is not or not contain any ID\", obj));");
			sw.println("}");

			sw.println("@Override");
			sw.println("public Subscriptor<%s, %s> subscriptor() {", daoMapping.proxy.getName(),
					daoMapping.id.getName());
			sw.println("	Class<%1$s> theClazz = %1$s.class;", daoMapping.proxy.getName());
			sw.println(
					"	return (this.subscriptor == null) ? (this.subscriptor = new RFSubscriptor<%s, %s, %s>(theClazz, this.getDataFactory().getEventBus(), this)) : this.subscriptor;",
					daoMapping.proxy.getName(), daoMapping.id.getName(), daoFactoryClass.getName());
			sw.println("}");

			for (JMethod method : this.dao.getMethods()) {
				writeDaoMethod(method, logger, context, rfDaoGeneratedSimpleClassname, sw, true);
			}

			sw.commit(logger);
		}
	}

	protected void writePlainDao(TreeLogger logger, GeneratorContext context, String rfDaoGeneratedSimpleClassname)
			throws UnableToCompleteException {
		SourceWriter sw = getRFPlainDaoSourceWriter(logger, context, rfDaoGeneratedSimpleClassname);

		if (sw != null) {
			DaoMapping daoMapping = this.getDaoMapping(logger);

			sw.println("public %s(RequestProvider<%s> useProvider, Map<String, RequestQueue> queueRequests) {",
					rfDaoGeneratedSimpleClassname, requestFactoryClass.getName());
			sw.println("	super(useProvider, queueRequests);");
			sw.println("}");
			sw.println();
			sw.println("@Override");
			sw.println("public %s getRequest(RequestProvider<%s> p) {", daoMapping.request.getName(),
					requestFactoryClass.getName());
			sw.println("	return p.getRequest(%s.class);", daoMapping.request.getName());
			sw.println("}");
			sw.println();

			for (JMethod method : this.dao.getMethods()) {
				writeDaoMethod(method, logger, context, rfDaoGeneratedSimpleClassname, sw, false);
			}

			sw.commit(logger);
		}
	}

	private SourceWriter getRFEntityDaoSourceWriter(TreeLogger logger, GeneratorContext ctx,
			String rfDaoGeneratedSimpleClassname) throws UnableToCompleteException {
		JPackage classPkg = rfDataManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		PrintWriter printWriter = ctx.tryCreate(logger, packageName, rfDaoGeneratedSimpleClassname);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				rfDaoGeneratedSimpleClassname);

		String[] imports = new String[] { RFDaoFactory.class.getName(), RequestProvider.class.getName(),
				Map.class.getName(), RequestQueue.class.getName(), daoFactoryClass.getQualifiedSourceName(),
				requestFactoryClass.getQualifiedSourceName(), EntityProxyId.class.getName(), Format.class.getName(),
				DaoOperationManager.class.getName(), RFDaoOperationManagerWrapper.class.getName(),
				RFBaseIdImpl.class.getName(), RFDaoReceiver.class.getName(), Subscriptor.class.getName(),
				RFSubscriptor.class.getName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		composerFactory.addImport(dao.getQualifiedSourceName());

		DaoMapping daoMapping = getDaoMapping(logger);

		composerFactory.setSuperclass(String.format("%s<%s,%s,%s,%s,%s>", RFBaseEntityDao.class.getName(),
				daoMapping.proxy.getName(), daoMapping.request.getName(), daoMapping.id.getName(),
				daoFactoryClass.getName(), requestFactoryClass.getName()));

		composerFactory.addImport(daoMapping.proxy.getQualifiedSourceName());
		composerFactory.addImport(daoMapping.request.getQualifiedSourceName());
		composerFactory.addImport(daoMapping.id.getQualifiedSourceName());

		composerFactory.addImplementedInterface(dao.getName());

		return composerFactory.createSourceWriter(ctx, printWriter);
	}

	private SourceWriter getRFPlainDaoSourceWriter(TreeLogger logger, GeneratorContext ctx,
			String rfDaoGeneratedSimpleClassname) throws UnableToCompleteException {
		JPackage classPkg = rfDataManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		PrintWriter printWriter = ctx.tryCreate(logger, packageName, rfDaoGeneratedSimpleClassname);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				rfDaoGeneratedSimpleClassname);

		String[] imports = new String[] { RFDaoFactory.class.getName(), RequestProvider.class.getName(),
				Map.class.getName(), RequestQueue.class.getName(), daoFactoryClass.getQualifiedSourceName(),
				EntityProxyId.class.getName(), requestFactoryClass.getQualifiedSourceName(),
				RFDaoOperationManagerWrapper.class.getName(), RFBaseIdImpl.class.getName(),
				RFDaoReceiver.class.getName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		for (JClassType dao : daos) {
			composerFactory.addImport(dao.getQualifiedSourceName());
		}

		DaoMapping daoMapping = getDaoMapping(logger);

		composerFactory.setSuperclass(String.format("%s<%s,%s,%s>", RFBaseDao.class.getName(),
				daoMapping.request.getName(), daoFactoryClass.getName(), requestFactoryClass.getName()));

		composerFactory.addImport(daoMapping.request.getQualifiedSourceName());

		composerFactory.addImplementedInterface(dao.getName());

		return composerFactory.createSourceWriter(ctx, printWriter);
	}

	protected DaoMapping getDaoMapping(TreeLogger logger) throws UnableToCompleteException {
		DaoMapping daoMapping = RFGeneratorBase.daoMapping.get(this.dao);

		if (daoMapping == null) {
			logger.log(
					TreeLogger.ERROR,
					Format.substitute("There is no mapping for the dao '{0}' on the class {1}", dao, rfDataManagerType),
					null);
			throw new UnableToCompleteException();
		}
		return daoMapping;
	}

	private void writeDaoMethod(JMethod method, TreeLogger logger, GeneratorContext context,
			String rfDaoGeneratedSimpleClassname, SourceWriter sw, boolean isEntityDao)
			throws UnableToCompleteException {
		sw.println("@Override");
		JParameterizedType returnParametrized = method.getReturnType().isParameterized();
		sw.print("public %s %s(", returnParametrized.getParameterizedQualifiedSourceName(), method.getName());
		boolean containsEntityId = writeParameters(method.getParameters(), logger, context,
				rfDaoGeneratedSimpleClassname, sw);
		sw.print(") {");
		sw.println();
		sw.indent();
		if (!containsEntityId) {
			writeSimpleDaoMethodBody(method, logger, context, sw);
		} else {
			writeWithEntitiesIdDaoMethodBody(method, logger, context, sw, isEntityDao);
		}
		sw.outdent();
		sw.println("}");
	}

	private boolean writeParameters(JParameter[] parameters, TreeLogger logger, GeneratorContext context,
			String rfDaoGeneratedSimpleClassname, SourceWriter sw) {

		JClassType jClassIsEntityId = context.getTypeOracle().findType(IsEntityId.class.getName());

		boolean containsEntityId = false;
		for (int i = 0; i < parameters.length; i++) {
			JParameter parameter = parameters[i];

			JClassType jClass = parameter.getType().isClassOrInterface();
			boolean isEntityId = (jClass != null && jClass.isAssignableTo(jClassIsEntityId));

			/*
			 * FIXME: Fixing an error on GWT 2.5.1 Super-Dev mode where only on
			 * the first compilation it works properly
			 */
			if (!containsEntityId && !isEntityId) {
				isEntityId = patchCheckIsEntityId(logger, jClass, jClassIsEntityId);
			}

			containsEntityId |= isEntityId;
		}

		for (int i = 0; i < parameters.length; i++) {
			if (i != 0) {
				sw.print(", ");
			}
			JParameter parameter = parameters[i];

			if (containsEntityId) {
				sw.print("final ");
			}

			sw.print(parameter.getType().getParameterizedQualifiedSourceName());
			sw.print(" ");
			sw.print(parameter.getName());
		}

		return containsEntityId;
	}

	protected boolean patchCheckIsEntityId(TreeLogger logger, JClassType jClass, JClassType jClassIsEntityId) {
		boolean isEntityId = false;
		JClassType[] implementedInterfaces = null;
		if (jClass != null && ((implementedInterfaces = jClass.getImplementedInterfaces()) != null)) {
			for (JClassType interfz : implementedInterfaces) {
				if (interfz.getQualifiedSourceName().equals(jClassIsEntityId.getQualifiedSourceName())) {
					isEntityId = true;
					if (!this.bugWarned) {
						logger.log(TreeLogger.WARN, "Bug still continues. Applying patch...", null);
						this.bugWarned = true;
					}
					break;
				}
			}
		}
		return isEntityId;
	}

	private void writeSimpleDaoMethodBody(JMethod method, TreeLogger logger, GeneratorContext context, SourceWriter sw)
			throws UnableToCompleteException {
		JParameter[] parameters = method.getParameters();
		DaoMapping daoMapping = getDaoMapping(logger);
		sw.println("%s req = this.getRequest(useProvider);", daoMapping.request.getName());
		sw.println();
		sw.print("return getRFOperationManagerWrapper(req.%s(", method.getName());
		for (int i = 0; i < parameters.length; i++) {
			if (i != 0) {
				sw.print(",");
			}
			JParameter parameter = parameters[i];
			sw.print(parameter.getName());
		}
		sw.println("));", method.getName());
	}

	/*
	 * FIXME: Cuando resuelvan el Issue 5525 accedemos con el EntityProxy
	 * directamente
	 */
	private void writeWithEntitiesIdDaoMethodBody(JMethod method, TreeLogger logger, GeneratorContext context,
			SourceWriter sw, boolean isEntityDao) throws UnableToCompleteException {
		JParameterizedType returnParametrized = method.getReturnType().isParameterized();
		sw.println("final RFDaoOperationManagerWrapper<%1$s> emptyOm = new RFDaoOperationManagerWrapper<%1$s>();",
				returnParametrized.getTypeArgs()[0].getParameterizedQualifiedSourceName());

		List<String> finalNames = new ArrayList<String>();

		writeFindProxyBody(0, method, logger, context, sw, finalNames, isEntityDao);

		sw.println("return emptyOm;");
	}

	private JType findProxyType(JType paramType, TreeLogger logger) throws UnableToCompleteException {
		for (JClassType entityDao : RFGeneratorBase.entityDaos) {
			DaoMapping daoInfo = daoMapping.get(entityDao);
			if (daoInfo.id.getQualifiedSourceName().equals(paramType.getQualifiedSourceName())) {
				return daoInfo.proxy;
			}
		}

		logger.log(TreeLogger.ERROR, String.format(
				"In order to able to use the %s entity id class you must annotate an entity dao for it on %s",
				paramType.getSimpleSourceName(), rfDataManagerType.getSimpleSourceName()), null);
		throw new UnableToCompleteException();
	}

	private void writeFindProxyBody(int startBy, JMethod method, TreeLogger logger, GeneratorContext context,
			SourceWriter sw, List<String> finalNames, boolean isEntityDao) throws UnableToCompleteException {
		JClassType jClassIsEntityId = context.getTypeOracle().findType(IsEntityId.class.getName());

		boolean foundEntityId = false;
		for (int i = startBy; i < method.getParameterTypes().length; i++) {
			JType paramType = method.getParameterTypes()[i];
			JClassType jClassType = paramType.isClassOrInterface();
			boolean isEntityId = jClassType != null && jClassType.isAssignableTo(jClassIsEntityId);

			/*
			 * FIXME: Fixing an error on GWT 2.5.1 Super-Dev mode where only on
			 * the first compilation it works properly
			 */
			if (!foundEntityId && !isEntityId) {
				isEntityId = patchCheckIsEntityId(logger, jClassType, jClassIsEntityId);
			}

			foundEntityId |= isEntityId;
			String paramName = method.getParameters()[i].getName();
			if (foundEntityId) {
				JType proxyType = findProxyType(paramType, logger);
				finalNames.add("proxy" + i);
				sw.println(
						"EntityProxyId<%1$s> proxy%2$dId = (%3$s != null) ? (EntityProxyId<%1$s>) ((%4$s) %3$s).getEntityProxyId() : null;",
						proxyType.getQualifiedSourceName(), i, paramName, RFBaseIdImpl.class.getName());
				sw.println();
				if (isEntityDao) {
					sw.println("pendingOpStart();");
				}
				sw.println();
				sw.println("RFDaoReceiver<%1$s> receiver%2$d = new RFDaoReceiver<%1$s>() {",
						proxyType.getQualifiedSourceName(), i);
				if (isEntityDao) {
					sw.println("	@Override");
					sw.println("	public void onEnd(boolean success) {");
					sw.println("		super.onEnd(success);");
					sw.println();
					sw.println("		pendingOpEnd();");
					sw.println("	}");
					sw.println();
				}
				sw.println("	@Override");
				sw.println("	public void success(final %s proxy%d) {", proxyType.getQualifiedSourceName(), i);
				sw.indent();
				sw.indent();
				sw.indent();
				writeFindProxyBody(i + 1, method, logger, context, sw, finalNames, isEntityDao);
				sw.outdent();
				sw.outdent();
				sw.outdent();
				sw.println("	}");
				sw.println("};");

				sw.println("if (proxy%dId != null) {", i);
				sw.println("	getDataFactory().find(proxy%1$dId).fire(receiver%1$d);", i);
				sw.println("} else {");
				sw.println("	receiver%1$d.onSuccess(null);", i);
				sw.println("}");
				break;
			} else {
				finalNames.add(paramName);
			}
		}

		if (!foundEntityId) {
			DaoMapping daoInfo = daoMapping.get(dao);
			JParameterizedType returnParametrized = method.getReturnType().isParameterized();

			sw.println("%s req = getRequest(useProvider);", daoInfo.request.getName());
			sw.println();
			sw.print("RFDaoOperationManagerWrapper<%s> om = getRFOperationManagerWrapper(req.%s(",
					returnParametrized.getTypeArgs()[0].getParameterizedQualifiedSourceName(), method.getName());
			for (int i = 0; i < finalNames.size(); i++) {
				String parameterName = finalNames.get(i);
				if (i != 0) {
					sw.print(",");
				}
				sw.print(parameterName);
			}
			sw.println("));");
			sw.println();
			sw.println("om.bindTo(emptyOm);");
		}

	}

}
