package org.ioc.commons.impl.gwt.rebind.dataaccess.requestfactory.generator;

import java.io.PrintWriter;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.RFDataManager;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFDaoFactory;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFNoTxRequestProvider;
import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFTxRequestProvider;
import org.ioc.commons.integration.dataaccess.dao.Tx;
import org.ioc.commons.utils.Format;
import org.ioc.commons.utils.FormatterLogger;

import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.TreeLogger.Type;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;

public class RFDataManagerGenerator extends RFGeneratorBase {

	static final String GENERATED_CLASS_SUFIX = "Generated";
	private String daoFactoryGeneratedName;

	public RFDataManagerGenerator() {
		// Nada
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);
		
		String rfDataManagerSimpleSourceName = rfDataManagerType.getSimpleSourceName();
		String rfDataManagerGeneratedFullClassname = typeName + GENERATED_CLASS_SUFIX;
		
		logger.log(Type.INFO, Format.substitute("Generating RequestFactory daos for {0} data manager.", rfDataManagerSimpleSourceName));

		SourceWriter sw = getRFDataManagerSourceWriter(logger, context);

		if (sw != null) {

			sw.println();
			sw.indent();
			sw.println("private static final FormatterLogger logger = FormatterLogger.getLogger(%s.class);",
					rfDataManagerSimpleSourceName + GENERATED_CLASS_SUFIX);
			sw.println();
			writeNewNonTransactionalDaoFactory(logger, context, sw);
			sw.println();
			writeOpenTransaction(logger, context, sw);
			sw.println();
			sw.outdent();

			sw.commit(logger);
		}

		return rfDataManagerGeneratedFullClassname;
	}

	private void writeNewNonTransactionalDaoFactory(TreeLogger logger, GeneratorContext context, SourceWriter sw) throws UnableToCompleteException {

		sw.println("@Override");
		sw.println("protected %s newNonTransactionalDaoFactory() {", daoFactoryClass.getSimpleSourceName());
		sw.indent();
		sw.println("assert this.isInitialized() : \"Data Manager must be initialized\";");

		this.daoFactoryGeneratedName = new RFDaoFactoryGenerator().generate(logger, context,
				RFDaoFactory.class.getSimpleName());
		String rfNoTxRequestProviderGeneratedName = new RFNoTxRequestProviderGenerator().generate(logger, context,
				RFNoTxRequestProvider.class.getSimpleName());

		sw.println("return new %s(new %s(dataFactory), queueRequests);", daoFactoryGeneratedName,
				rfNoTxRequestProviderGeneratedName);
		sw.outdent();
		sw.println("}");
	}

	private void writeOpenTransaction(TreeLogger logger, GeneratorContext context, SourceWriter sw) throws UnableToCompleteException {

		sw.println("@Override");
		sw.println("public Tx<%s> openTransaction(String transactionName) {", daoFactoryClass.getSimpleSourceName());
		sw.indent();
		sw.println("assert this.isInitialized() : \"Data Manager must be initialized\";");

		String rfTxRequestProviderGeneratedName = new RFTxRequestProviderGenerator(this.daoFactoryGeneratedName).generate(logger, context,
				RFTxRequestProvider.class.getSimpleName());

		sw.println("%s tx = new %s(this.dataFactory, transactionName, queueRequests, bindedProxiesMap);",
				rfTxRequestProviderGeneratedName, rfTxRequestProviderGeneratedName);
		sw.println();
		sw.println("logger.info(\"Transaction '{0}' opened\", tx);");
		sw.println();
		sw.println("return tx;");
		sw.outdent();
		sw.println("}");
	}

	protected SourceWriter getRFDataManagerSourceWriter(TreeLogger logger, GeneratorContext ctx) {
		JPackage classPkg = rfDataManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		String generatedClassSimpleName = rfDataManagerType.getName() + GENERATED_CLASS_SUFIX;
		PrintWriter printWriter = ctx.tryCreate(logger, packageName, generatedClassSimpleName);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				generatedClassSimpleName);

		String[] imports = new String[] { RFDataManager.class.getName(), Tx.class.getName(),
				FormatterLogger.class.getName(), daoFactoryClass.getQualifiedSourceName(),
				requestFactoryClass.getQualifiedSourceName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		composerFactory.setSuperclass(rfDataManagerType.getQualifiedSourceName());

		return composerFactory.createSourceWriter(ctx, printWriter);
	}
}
