package org.ioc.commons.impl.gwt.rebind.dataaccess.requestfactory.generator;

import java.io.PrintWriter;

import org.ioc.commons.impl.gwt.integration.client.dataaccess.requestfactory.rebind.internal.RFNoTxRequestProvider;
import org.ioc.commons.utils.Format;

import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.JPackage;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;
import com.google.web.bindery.requestfactory.shared.RequestContext;

public class RFNoTxRequestProviderGenerator extends RFGeneratorBase {

	public RFNoTxRequestProviderGenerator() {
		// Nada
	}

	@Override
	public String generate(TreeLogger logger, GeneratorContext context, String typeName)
			throws UnableToCompleteException {
		super.generate(logger, context, typeName);
		String rfNoTxRequestSimpleClassname = typeName + RFDataManagerGenerator.GENERATED_CLASS_SUFIX;
		String rfNoTxRequestFullClassname = rfDataManagerType.getPackage().getName() + "."
				+ rfNoTxRequestSimpleClassname;

		SourceWriter sw = getRFNoTxRequestProviderSourceWriter(logger, context, rfNoTxRequestSimpleClassname);

		if (sw != null) {

			sw.println("public %s(%s dataFactory) {", rfNoTxRequestSimpleClassname, requestFactoryClass.getName());
			sw.println("	super(dataFactory);");
			sw.println("}");
			sw.println();
			sw.println("@Override");
			sw.println("public <R extends RequestContext> R getRequest(Class<R> clazz) {");
			sw.indent();
			boolean first = true;
			for (JClassType requestClass : requests) {
				if (!first) {
					sw.print(" else ");
				}
				sw.println("if (clazz.equals(%s.class)) {", requestClass.getSimpleSourceName());
				sw.println("		return (R) this.get%s();", requestClass.getSimpleSourceName());
				sw.println("}");
				first = false;
			}
			sw.println("		throw new IllegalArgumentException(\"The RequestContext \" + clazz + \" has not been registered. Add a new method on class %s returning this request context\");",requestFactoryClass.getName());
			sw.outdent();
			sw.println("}");
			sw.println();
			for (JClassType requestClass : requests) {
				sw.println("public %1$s get%1$s() {", requestClass.getSimpleSourceName());
				sw.println("	return this.dataFactory.%s();", Format.decapitalize(requestClass.getSimpleSourceName()));
				sw.println("}");
			}
			sw.commit(logger);
		}

		return rfNoTxRequestFullClassname;
	}

	private SourceWriter getRFNoTxRequestProviderSourceWriter(TreeLogger logger, GeneratorContext ctx,
			String rfNoTxRequestSimpleClassname) throws UnableToCompleteException {
		JPackage classPkg = rfDataManagerType.getPackage();
		String packageName = classPkg == null ? "" : classPkg.getName();

		PrintWriter printWriter = ctx.tryCreate(logger, packageName, rfNoTxRequestSimpleClassname);
		if (printWriter == null) {
			return null;
		}

		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName,
				rfNoTxRequestSimpleClassname);

		String[] imports = new String[] { RequestContext.class.getName(), RFNoTxRequestProvider.class.getName(),daoFactoryClass.getQualifiedSourceName(), requestFactoryClass.getQualifiedSourceName() };
		for (String imp : imports) {
			composerFactory.addImport(imp);
		}

		composerFactory.setSuperclass(String.format("%s<%s,%s>", RFNoTxRequestProvider.class.getName(),
				daoFactoryClass.getName(), requestFactoryClass.getName()));

		for (JClassType requestClass : requests) {
			composerFactory.addImport(requestClass.getQualifiedSourceName());
		}

		return composerFactory.createSourceWriter(ctx, printWriter);
	}
}
